#!/usr/bin/perl -w
# item.pl  -- Mako Hill <mako@debian.org>
# written for Organizers' Collaborative -- 2001
# this code is licensed unde the GNU GPL, see COPYING for more details

# this script works on item as individual units. pretty much any
# action taken on only ONE list will be found in this script. The
# candidates for moving outside of this are the two submit lists which
# don't deal with code the same way. However because they share so
# much code with editing, it makes sense to house them in the same
# place and share functions.

use strict;
use CGI qw/:standard/;             # used for cookies and params
use Calf::Auth qw( :base );          # Site authentication module
use Calf::Config qw( :base :item );  # Site Config module
use Calf::Utils qw( :base :template :db :mail ); # lots of site utils

## Open handlers to CGI and the database
my $dbh = Calf::Utils::open_db;
my $sth;
my $q = new CGI;

## Put the input data passed through the previous form into a hash
my $form_info = $q->Vars;

# Set up the important early variables
my $session_info = &valid_id( $q, $dbh ); # is the person logged in?
my $action = $form_info->{action};        # which action is requested?
my $item = $form_info->{item};            # which list are we working on?
my $i;                                    # for use in looping
$| = 1;

# functions open to editors and non-editors alike
if ($form_info->{action} eq 'newlist')
  { &newlist() }
elsif ($form_info->{action} eq 'commitnew')
  { &commitnew() }
elsif ($form_info->{action} eq 'view')
  { &view() }

# First, we need to need to block non-editors from editor-only
# functions
elsif (defined $session_info) {              # this person is a valid editor

  if ($form_info->{action} eq 'edit')
    { &edit() }                              # edit the page
  elsif ($form_info->{action} eq 'change')
    { &change() }                            # commit changes from an edit page
  elsif ($form_info->{action} eq 'approve')
    { &approve() }                           # just approve the list
  elsif ($form_info->{action} eq 'delete')
    { &remove() }                            # remove the list all-together
  elsif ($form_info->{action} eq 'reject')
    { &reject() }
}
# if there are no bites, just fail gracefully telling the user this
else { &error( 'none', 'invalidaction' ) }

# disconnect from the database and clean up.. :) We're done baby.
$dbh->disconnect;


###########################################################
## ACTION SUBROUTINES
###########################################################

## view - view pages
##### args: none
sub view {
  # fail immediately if there is no item given (can't view NOTHING)
  &error( 'view', 'noitem' ) unless (defined $item);

  # declare/set variables used later in this script
  my ( $editor, $editor_thislist );
  my ( $type );
  my $descriptions = &build_descriptions( $dbh ); # build descriptions for TT
  my $list = &grab_listinfo( $item, $dbh );       # grab list from the database

  if ( defined( $session_info ) ) {      # if the person is logged in..
    $editor = 1;                         # mark them as an editor
    # verify if they are an editor of this list
    $editor_thislist = &verify_editorship( $list->{id}, $session_info, $dbh );
  }

  # copy data into the outgoing hash
  my $vars = { session_info => $session_info,
	       editor => $editor,
	       editor_thislist => $editor_thislist,
	       list => $list,
	       descriptions => $descriptions,
	       page => "view"
	     };

  # process the cookie and print out the page
  &process_cookie( $session_info->{session_id} );
  &process_template('item/view.html', $vars );
}

## edit - RESPONSE TO AN EDITORS ONLY REQUEST TO EDIT AN LIST
##### args: but item must be set of course
sub edit {

  # fail immediately if there is no item given (can't view NOTHING)
  &error( 'view', 'noitem' ) unless (defined $item);

  # declare/set vars
  my ( $editor, $editor_thislist );
  my $list = &grab_listinfo( $item, $dbh ); # get list info from the database

  if ( defined( $session_info ) ) {         # if the person is logged in..
    $editor = 1;                            # mark them as an editor
    # verify if they are an editor of this list
    $editor_thislist = &verify_editorship( $list->{id}, $session_info, $dbh );
  }

  # exit right here if the person is not an editor (this should never
  # actually happen as the code before this catch it)
  (&invalid_access && return)
    unless defined( $editor_thislist );

  # cycle through each editable $field to run a series of tests. we
  # have to use a for instead of a foreach because we will be tossing
  # things out of the list we are iterating through while we test it
  for ($i=0; $i <= $#$editable_fields; $i++) {
    my $field = $editable_fields->[$i];

    # we'll first off, just set whatever in the database to the
    # field->{value} and then do checking.. this is not 100% efficient
    # but it seems most straight forward to do it this way, at least
    # at this point

    $field->{value} = $list->{"$field->{name}"};

    # we toss fields marked submitonly
    if (defined $field->{submitonly} && $field->{submitonly} == 1)
      {	splice( @$editable_fields, $i, 1);
	--$i; next; }

    # for all those marked categorization (right now only the issues
    # and constit fields), we need to do some special tricks so we
    # pass the whole list of checkbox fields as part of the field
    # definition
    if ($field->{type} eq 'categorization') {

      # we grab all the categories out of the db with &make_type_hash
      my $all_categories = &make_type_hash( $field->{name}, $dbh );

      # then we cycle through the list of all the possible categories,
      # pushing each one into the a list of categories that will make
      # up the checkbox options.
      #
      # we test each possible code to see if it matches one of the
      # codes for the current list, it is does, we mark it as checked
      # before we push it in
      my ($code, @processed_categories);

      foreach $code (keys %$all_categories) {
	if (grep { $code eq $_ } @{ $field->{value} })
	  { push @processed_categories, { code => $code,
					  description =>
					    $all_categories->{$code},
					  checked => 1 }; }
	else
	  { push @processed_categories, { code => $code,
	        description => $all_categories->{$code} }; }
      }

      # we sort the list alphabetically by description
      @processed_categories =
	sort( { $a->{description} cmp $b->{description} }
	      @processed_categories );

      # finally we assign the sorted list to its field.. good to go
      $field->{checkboxoptions} = \@processed_categories;
    }
  }

  # we make the necessary variables available to the templating system..
  my $vars = { session_info => $session_info,
	       editor => $editor,
	       editor_thislist => $editor_thislist,
	       fields => $editable_fields,
	       list => $list,
	       page => "edit" };

  # and we process away....
  &process_cookie( $session_info->{session_id} );
  &process_template('item/edit.html', $vars );
}

## change -- THE RESPONSE TO AN EDITOR CLICKING ON THE CHANGE BUTTON ON THE
## EDIT PAGE
##### args: none
sub change {

  # declare/set vars
  my ( $editor, $editor_thislist );
  my ($field, $list_new, $fatal_error, $errors, $req_error ) = ();
  my $list_current = &grab_listinfo( $form_info->{item}, $dbh );
  my $descriptions = &build_descriptions( $dbh );

  # authentication stuff
  if ( defined( $session_info ) ) {            # if the person is logged in..
    $editor = 1;                               # mark them as an editor
    # verify if they are an editor of this list
    $editor_thislist = &verify_editorship( $list_current->{id},
					   $session_info, $dbh );
  }
  (&invalid_access && return) 
    unless defined( $editor_thislist );

  # we cycle through this in a way that mirrors the code about in edit
  # we have to use a for instead of a foreach because we will be
  # tossing things out of the list we are iterating through while we
  # test it
  for ($i=0; $i <= $#$editable_fields; $i++) {
    $field = $editable_fields->[$i];

    if (defined $field->{submitonly})     # toss out anything marked as submit-
      { splice( @$editable_fields, $i, 1);# only as it is irrelavant
	--$i; next; }
  }

  # process incoming fields munging and checking each field and then marking
  # them as accurate, or inacrate and reporting any fatal errors
  ($list_new, $errors, $fatal_error, $req_error) =
    &process_incoming_fields( $editable_fields, $list_current );

  # now that we down the testing, we determine a course of action

  # fail if there are fatal errors (stuff that people shouldn't be
  # able with the interfaces we provide
  if (defined $fatal_error) { &error( 'change', 'badinput' ); }

  # with req_errors, we ask for things to be fixed (ALWAYS)
  elsif (defined $req_error) {

    # set up up the variables for TT
    my $vars = { session_info => $session_info,
		 req_error => $req_error,
		 errors => $errors,
		 fields => $list_new,
		 list_old => $list_current,
		 descriptions => $descriptions,
		 page => "change"
	       };

    # process the cookie and print the page
    &process_cookie( $session_info->{session_id} );
    &process_template('item/checkagain.html', $vars );
    exit;          # there will be hits below if we don't leave now :)
  }

  # we ask for fixing if there are normal (not in required fields
  # and not fatal) errors the first time through only
  elsif ((defined $errors) && (not defined $form_info->{checked})) {

    # set up up the variables for TT
    my $vars = { session_info => $session_info,
		 errors => $errors,
		 fields => $list_new,
		 list_old => $list_current,
		 descriptions => $descriptions,
		 page => "change"
	     };

    # process the cookie and print the page
    &process_cookie( $session_info->{session_id} );
    &process_template('item/checkagain.html', $vars );
    exit;          # there will be hits below if we don't leave now :)
  }

  # (second is defined, so are errors) or (errors is not defined) and
  # there are no fatal errors.. good to go so we input it and return a
  # success!.. whohoo
  else {
    # declare and start setting up our update statement
    my $update_statement;
    $update_statement  = qq/ UPDATE calf_data SET /;
    $update_statement .= qq/ updated = NOW(), /;
    $update_statement .= qq/ lasteditor = "$session_info->{username}", /;

    # we cycle through each list again, same as before
    for ($i=0; $i <= $#$list_new; $i++ ) {
      $field = $list_new->[$i];

      # we set empty values to NULL
      if (defined $field->{value} && $field->{value} =~ /^\s*$/)
	{ $field->{value} = 'NULL'; }

      else {
	# we need to create the appropriate "test for the lastchange
	# field in teh database. in this case, the options are either
	# "edit & approve" or just "edit"
	if ($field->{name} eq 'approved'        # if the field is correct
	    && $field->{value} == 1             # and its set to "yes"
	    && $list_current->{approved} == 0 ) # and it's not already set
	  { $update_statement .= qq/ lastchange = "edit & approve" , /;	}
	else
	  # otherwise, this is just a normal edit
	  { $update_statement .= qq/ lastchange = "edit" , /; }

	# non-null values we need to quote for db entry
	$field->{value} = $dbh->quote($field->{value});
      }

      # then we just set the print the name = value part and then a
      # comma unless its the last one
      $update_statement .= qq{ $field->{name} = $field->{value} };
      $update_statement .= " , " unless $i == $#$list_new;
    }

    # after all the set statements, we make sure it's acting on the right list
    $update_statement .= qq{ WHERE id = "$list_current->{id}" };

    $dbh->do("$update_statement")        # finally we actually insert the data
      || &db_error( 'do', $!, $update_statement );  # and fail gracefully :)

    my $list = &grab_listinfo( $item, $dbh );# we need the new info from thedb
                                             # to show to people their changes

    # set up the data we want to hand to the template
    my $vars = { session_info => $session_info,
		 list => $list,
		 item => $form_info->{item},
		 editor => $editor,
		 editor_thislist => $editor_thislist,
		 descriptions => $descriptions,
		 page => "change"
	       };

    # build the cookie and print our HTML
    &process_cookie( $session_info->{session_id} );
    &process_template('item/submit_success.html', $vars );
  }
}

## newlist -- this will genereate the page that people see when they
## want to submit a link
##### args: none
sub newlist {
  my ( $editor, $editor_thislist );
  if ( defined( $session_info ) )      # if the person is logged in..
    { $editor = 1; }                   # mark them as an editor
  # there's no point in going further, people can't be an editor for
  # this listas it doesn't exist yet

  # like in &edit, we cycle through the editable fields
  for ($i=0; $i <= $#$editable_fields; $i++) {
    my $field = $editable_fields->[$i]; # the field we are working on

    # remove editor only fields from this page
    if (defined $field->{editoronly})
      { splice( @$editable_fields, $i, 1);
	--$i; next; }

    # set the default values for name and email if the person is an
    # editor and we already have that info
    if (defined $editor)  {
      if ( $field->{name} eq 'contactname' ) {
	$field->{value} = $session_info->{fullname};
      } elsif ($field->{name} eq 'contactemail') {
	$field->{value} = $session_info->{email};
      }
    }

    # if we are dealing with a "type" we need to do special voodoo
    if ($field->{type} eq 'categorization') {
      # set and reset variables we'll need
      my ($code, @processed_categories) = ();
      # we grab all the categories out of the db with &make_type_hash
      my $all_categories = &make_type_hash( $field->{name}, $dbh );

      # build/sort the code/descriptions we'll need for the checkbox options
      # check out the docs in the edit sub for more info
      foreach $code (keys %$all_categories) {
	push @processed_categories, { code => $code, 
				      description => $all_categories->{$code}};
      }

      @processed_categories = sort( { $a->{description} cmp $b->{description} }
				    @processed_categories );
      $field->{checkboxoptions} = \@processed_categories;
    }
  }

  # We make the necessary variables available to the templating system..
  my $vars = { session_info => $session_info,
	       editor => $editor,
 	       fields => $editable_fields,
	       page => "newlist"
	     };

  # and we process away....
  &process_cookie( $session_info->{session_id} ) if (defined $editor);
  &process_template('item/edit.html', $vars );
}

## commitnew - this will actually SUBMIT (or at least try) a new list
## passed. it's like the second step of newlist
##### args: none
sub commitnew {
  # set, reset, and declare vars for this sub
  my ( $editor, $editor_thislist );
  my ( $field, $type, $code );
  my ( $list_new, $fatal_error, $errors, $req_error ) = ();
  my $descriptions = &build_descriptions ( $dbh );

  if ( defined( $session_info ) ) {       # if the person is logged in..
    $editor = 1;                          # mark them as an editor

    # we now need to find out if this editor is an editor for this
    # list they are trying to submit
    foreach $type ( @types ) {
      # grab the editors code for the type being worked on
      my $editor_codes = ( &retrieve_editor_codes( $session_info,
						   $type, $dbh ) );
      # go through and test each code to see if the editor is a match
      foreach $code (keys %$editor_codes ) {
	if ( $form_info->{"$type"} =~ m/$code/ )
	  { $editor_thislist = 1; last; }     # if yes, mark the editor/stop
      }
      if (defined $editor_thislist) { last; } # stop if there was success
    }                                         # on hit is enough
  }

  # just like in all the cases above, start iterating through the
  # editable fields, refer to comments in sections above
  for ($i=0; $i <= $#$editable_fields; $i++) {
    my $field = $editable_fields->[$i];

    if (defined $field->{editoronly})
      { splice( @$editable_fields, $i, 1);
	--$i; next; }
  }

  # one field is left blank becuase there is nothing to check against
  # here
  ( $list_new, $errors, $fatal_error, $req_error ) =
    &process_incoming_fields( $editable_fields, {} );

  # stop if there is a fatal error
  if ( defined $fatal_error ) { &error('commitnew', 'badinput' ); }

  # we will prompt people to check there data agin if the yhave not
  # checked it yet, regardless of errors -- all new lists are doublechecked
  elsif (not defined $form_info->{checked}) {
    my  $vars = { session_info => $session_info,
		  editor => $editor,
		  req_error => $req_error,
		  errors => $errors,
		  fields => $list_new,
		  descriptions => $descriptions,
		  page => 'commitnew'
		};

    &process_cookie( $session_info->{session_id} );
    &process_template('item/checkagain.html', $vars );
    exit;
  }

  # we will checkagain if there are any required errors ALWAYS
  elsif (defined $req_error) {
    my  $vars = { session_info => $session_info,
		  editor => $editor,
		  req_error => $req_error,
		  errors => $errors,
		  fields => $list_new,
		  descriptions => $descriptions,
		  page => 'commitnew'
		};

    &process_cookie( $session_info->{session_id} );
    &process_template('item/checkagain.html', $vars );
    exit;

  }

  # otherwise, we've got only permissiable (no) errors and we go ahead
  # with the inset into the database
  else {
    my ( %list ); # this will be needed so we can display something
                  # when we are done

    # declare and start setting up our insert statement
    my $insert_statement;
    $insert_statement  = qq/ INSERT INTO calf_data SET /;

    # we now need to fill in some info that wasn't handed to us by the
    # the last script but that we can find out on our own
    $insert_statement .= qq/ entered = NOW(), /;  # the list is being entered
    $insert_statement .= qq/ updated = NOW(), /;  # it is also being updated

    # if the person is an editor of this list or a superuser (close
    # enough) the list should be approved and the change should be
    # marked in the database
    if ( defined $editor_thislist || $session_info->{superuser}  == 1 ) {
      $insert_statement .= qq/ approved = 1, /;
      $insert_statement .= qq/ lasteditor = "$session_info->{username}", /;
      $insert_statement .= ( qq/ lastchange = / .
			     $dbh->quote("submit & approve") . " , " );
    }
    # if not, the list is unapproved and the lastchange is a submit
    else {
      $insert_statement .= qq/ approved = 0, /;
      $insert_statement .= ( qq/ lastchange = / .
			     $dbh->quote("submit") . " , " );
    }

    # we cycle through the editable fields once more, this time
    # building the SQL statement
    for ($i=0; $i <= $#$list_new; $i++ ) {
      $field = $list_new->[$i];

      # we copy the value so it can be displayed when we are done
      $list{"$field->{name}"} = $field->{value};

      # we set empty values to NULL to make database queries easier later
      if (defined $field->{value} && $field->{value} =~ /^\s*$/)
	{ $field->{value} = 'NULL'; }
      # non-null values we need to quote any nasty characters that
      # would mess up our SQL string
      else
	{ $field->{value} = $dbh->quote( $field->{value} ); }

      # finally, we print the field = value pair into our statement
      $insert_statement .= qq{ $field->{name} = $field->{value} };
      $insert_statement .= " , " unless $i == $#$list_new;
    }
    # finally we actually insert the data
    $dbh->do("$insert_statement")
      || &db_error( 'do', $!, $insert_statement ); # or die nicely

    # before we can pass %list to the template, we need to make sure
    # the $types will be displayable by the template so we need to
    # parse it out of a string into an array
    foreach $type (@types) {
      my @codes = &parse_codes( $list{"$type"} );
      $list{"$type"} = \@codes;
    }

    # now we put together our data and we're off
    my $vars = { session_info => $session_info,
		 list => \%list,
		 editor => $editor,
		 editor_thislist => $editor_thislist,
		 descriptions => $descriptions,
		 page => "createnew",
		 sth => $insert_statement
	       };

    &process_cookie( $session_info->{session_id} );
    &process_template('item/submit_success.html', $vars );
  }
}

## approve -- approve a list (only)
#### args: none
sub approve {
  # set vars and grab listinfo
  my ( $editor, $editor_thislist );
  my ( %headers, %mailvars );
  my ( $update_statement );
  my $list = &grab_listinfo( $item, $dbh ); # get list info from the database

  if ( defined( $session_info ) ) {         # if the person is logged in..
    $editor = 1;                            # mark them as an editor
    # verify if they are an editor of this list
    $editor_thislist = &verify_editorship( $item, $session_info, $dbh );
  }

  # exit right here if the person is not an editor (this should never
  # actually happen as the code before this catch it)
  (&invalid_access && return)
    unless defined( $editor_thislist );

  # notify the submitter it's approved.  Set up email template
  # NEED TO ADD ERRORCHECKING TO BYPASS THIS IF CONTACTEMAIL=""
  # rac 12-13

  %headers = ( From => ( "\"$session_info->{fullname}\" "
			 . "<$session_info->{email}>" ),
	       To => "$list->{contactemail}",
	       Subject => ( "\"$list->{name}\" was approved by "
			    . "an editor at DemocracyGroups.org" ) );

  # now we need to set the variables passed to the mail template
  %mailvars = ( contactname => $list->{contactname},
		name => $list->{list},
		id => $list->{id},
		listdescription => $list->{description},
		editorname => $session_info->{fullname},
		editoremail => $session_info->{email}	);

  # now we get down the business and send the email
  &send_email( \%headers, 'approve.email', \%mailvars );

  # this little statement should do the trick
  $update_statement = qq/ UPDATE calf_data SET
                          lastchange = "approve", approved = 1, updated = NOW(),
                          lasteditor = "$session_info->{username}"
                          WHERE id = $item /;

  # process the statement and fail nicely if needed
  $dbh->do( $update_statement )
    || &db_error( 'do', $!, $update_statement );

  # set the vars and process away
  my $vars = { session_info => $session_info,
	       list => $list,
	       editor => $editor,
	       editor_thislist => $editor_thislist,
	       descriptions => &build_descriptions( $dbh ),
	       page => "approve"
	     };
  &process_cookie( $session_info->{session_id} );
  &process_template('item/submit_success.html', $vars );
}

## remove -- deletes a list from the database ( i named it remove to
## avoid conflicting with the delete function in perl ).. byebye.. sniffle
sub remove {
  my ( $editor, $editor_thislist );
  my $db_statement;
  my $list = &grab_listinfo( $item, $dbh ); # get list info from the database

  if ( defined( $session_info ) ) {      # if the person is logged in..
    $editor = 1;                         # mark them as an editor
    # verify if they are an editor of this list
    $editor_thislist = &verify_editorship( $item, $session_info, $dbh );
  }

  # exit right here if the person is not an editor (this should never
  # actually happen as the code before this catch it)
  (&invalid_access && return)
    unless defined( $editor_thislist );

  # the GRAVEDIGGER will copy the list info into the
  # calf_data_graveyard first
  $db_statement = qq/ INSERT INTO calf_data_graveyard SELECT * FROM calf_data
                      WHERE calf_data.id = $item /;
  $dbh->do( $db_statement )
    || &db_error( 'do', $!, $db_statement );

  # now, with the list safely saved, we can delete it
  $db_statement = qq/ DELETE FROM calf_data WHERE id = $item /;
  $dbh->do( $db_statement )
    || &db_error( 'do', $!, $db_statement );

  # not much to pass in vars this time eh?, lets clean up and be done with it
  my $vars = { session_info => $session_info,
	       list => $list,
	       page => 'remove'
	     };

  &process_cookie( $session_info->{session_id} );
  &process_template('item/delete_success.html', $vars );

}

## reject - this is a lot of like delete except that it offers to send
## an email first.. i suppose this could pretty easily be combind into
## one nice sub called with different arguments.. that's a project for
## when i am not leaving in a day! :)
sub reject {
  # set/declare vars
  my ( $editor, $editor_thislist );
  my ( %headers, %mailvars );
  my $db_statement;
  my $list = &grab_listinfo( $item, $dbh ); # get list info from the database

  if ( defined( $session_info ) ) {         # if the person is logged in..
    $editor = 1;                            # mark them as an editor
    # verify if they are an editor of this list
    $editor_thislist = &verify_editorship( $item, $session_info, $dbh );
  }

  # exit right here if the person is not an editor (this should never
  # actually happen as the code before this catch it)
  (&invalid_access && return)
    unless defined( $editor_thislist );

  # if people have checked that they are sure and it says to email, lets email
  if ($form_info->{checked} == 1 && $form_info->{emailed} == 1) {
    # first lets the headers to what we want
    %headers = ( From => ( "\"$session_info->{fullname}\""
			   . "<dgroups\@organizenow.net>" ),
		 To => "$form_info->{recip1}",
		 Subject => ( "\"$list->{name}\" was rejected by "
			      . "an editor at DemocracyGroups.org" )
	       );

    # set the cc if people specified one
    $headers{'Cc'} = "$form_info->{recip2}"
      if ( (defined $form_info->{recip2})
	   && ($form_info->{recip2} !~ m/^\s*$/) );

    # now we need to set the variables passed to the mail template
    %mailvars = ( contactname => $list->{contactname},
		  name => $list->{list},
		  orgdescription => $list->{description},
		  reason => $form_info->{reason},
		  editorname => $session_info->{fullname},
		  editoremail => $session_info->{email}	);

    # now we get down the business and send the email
    &send_email( \%headers, 'reject.email', \%mailvars );
  }

  # if the person has gone through the first level, we can go ahead and 
  # delete the list
  if ( $form_info->{checked} == 1 ) {
    # copy the list into the graveyard
    $db_statement = qq/ INSERT INTO calf_data_graveyard SELECT * FROM calf_data
                        WHERE calf_data.id = $item /;
    $dbh->do( $db_statement )
      || &db_error( 'do', $!, $db_statement );

    # delete the list from the db
    $db_statement = qq/ DELETE FROM calf_data WHERE id = $list->{id} /;
    $dbh->do( $db_statement )
      || &db_error( 'do', $!, $db_statement );

    # we process the template and are finished
    my $vars = { session_info => $session_info,
		 list => $list,
		 recip1 => $form_info->{recip1},
		 recip2 => $form_info->{recip2},
		 emailed => $form_info->{emailed},
		 page => 'reject'
	       };

    &process_cookie( $session_info->{session_id} );
    &process_template('item/delete_success.html', $vars );
  }

  # if the user hasn't gone past the check, we need to ask to ask for
  # rejction info
  else {
    # this is the default rejection text, probably not stored in a ver
    # good way :)

    my $rejection_default = (
      qq/[ ] inaccuracy of the information\n/ .
      qq/[ ] the list is not related to social concerns\n/ .
      qq/[ ] the list does not appear to exist\n/ .
      qq/[ ] Other:  /   );

    my $vars = { session_info => $session_info,
		 list => $list,
		 descriptions => &build_descriptions( $dbh ),
		 page => "reject",
		 rejection_reason => $rejection_default
	       };

    &process_cookie( $session_info->{session_id} );
    &process_template('item/reject.html', $vars );

  }
}

## error -- a little wrapper around the dg_error script
##### args: subroutine_name, reason, details
sub error
  { &dg_error( 'script', 'item', @_ ); }


### GENERAL PURPOSE SUBROUTINES used within this script
##################################################################

## this is called to validate the "values" inside "fields" in the form of ##
## the $editible_fields data in Calf::Config.pm                             ##
sub process_incoming_fields {
  my ( $list_unsafe, $list_current ) = @_;
  my ( $errors, $fatal_error, $req_error );
  my $field;

  # we cycle through the list of fields and operate on each one
  for ($i=0; $i <= $#$list_unsafe; $i++) {
    $field = $list_unsafe->[$i];

    $field->{valid} = 0;               # each field defaults to invalid and
                                       # each value is is field with what the
                                       # user input from the form
    $field->{value} = $form_info->{"$field->{name}"};
    undef( $field->{value} ) if ( $field->{value} eq '' );

    # rich changed this and it's not what i want.. we still want to be
    # checking to make sure that people editing lists cant' change the
    # address to one that is already in the database. this probably
    # isn't a very pressing problem
    if ( $field->{name} eq 'listadd' && not $form_info->{action} eq 'change')
      { &check_repeats( $field->{name}, $field->{value} ); }

    # take only first 255 characters of comment field to avoid FATAL error
    if ( $field->{name} eq 'description')
    { $field->{value} = sprintf("%.255s", $field->{value} ); }

    # this works to take into the account the fact that selects return
    # nothing by default (maybe it can be avoided in javascript)
    if ($field->{type} eq 'select' && $field->{value} eq '')
      { $field->{value} = $list_current->{"$field->{name}"}; }

    # we need to do black magic on the Issues/Constits because of the
    # rather horrible way they stored
    if ($field->{type} eq 'categorization') {
      my @codes = ($q->param( "$field->{name}" ));  # new codes
      my $all_categories = &make_type_hash( $field->{name}, $dbh );
      my ($code, @processed_categories);

      ( $fatal_error = 1 )  ### **I'm not sure this protecting against **
	                    ### **everything and I should find out **
	if ( (($#codes + 1) * 2) > $field->{size} );

      # we do pretty much the same thing we did above in edit, see comments
      # there for more info
      foreach $code (keys %$all_categories) {
	if (grep { $code eq $_ } @codes) {
	  push @processed_categories,
            { code => $code, description => $all_categories->{$code},
	      checked => 1 };
	} else {
	  push @processed_categories, 
	    { code => $code, description => $all_categories->{$code} };
	}
      }

      # sort the categories by description
      @processed_categories =
	sort( { $a->{description} cmp $b->{description} }
	      @processed_categories );
      $field->{checkboxoptions} = [ @processed_categories ];

      undef $field->{value};        # the field value is going to be wrong
      foreach ( sort @codes )       # so we blank it and rebuild it here
	{ $field->{value} .= $_; }  # this will be possible to post intodb

      # if the category evaluates to true, just say it's valid. this
      # is NOT a good test but it's all i have time for right now.. :(
      if ($field->{value})
	{ $field->{valid} = 1; next; }
    }

    # i'm not sure how to check this intelligently but i know it needs
    # a special case to work properly.. see the next comment up
    elsif ($field->{type} eq 'radio') {
      $field->{valid} = 1;
      next;
    }

    # such a hack, i'm SO SO SORRY.. :)
    elsif ( $field->{required} == 1 )
      { ($field, $fatal_error) = &validate_incoming_field ($field); }

    # if the proposed value is the same as the current value.. call it valid
    # there's a lot of invalid data in the db that we can't hold editors
    # responsible for
#    elsif ($list_current->{"$field->{name}"} eq $field->{value})
    elsif ( ( defined $list_current->{"$field->{name}"}
	      && defined $field->{value}
	      && $list_current->{"$field->{name}"} eq $field->{value} )
	    || not( defined $list_current->{"$field->{name}"}
		    || defined $field->{value} )
	  )
	   { $field->{valid} = 1; next; }

    # validiate all remaining fields using the validate code in the
    # subroutine immediately below
    else {
      ($field, $fatal_error) = &validate_incoming_field ($field);
    }

    # now we look at the errors we returned for each item and see if
    # this changes the values we are going to return
    if (defined $fatal_error)      # if there are fatal errors, we stop right
      { last; }                    # here
    if ($field->{valid} == 0)      # if the field is still invalid here, we
      { ++$errors; }               # count that as an error
    if ($field->{required} == 1) {
      if ( ($field->{valid} == 0)       # finally, if there is a non-fatal
	   || ($field->{value} eq '')   # error in a required field
	   || ($field->{value} eq undef) )
	{ ++$req_error;                         # we mark that as a $req_error
	  if ( $field->{valid} == 1 )
	     { $field->{valid} = 0; ++$errors; } 
	}
                                                # update this as a
                                                # real error if the
                                                # field is otherwise
                                                # valid
    }
    next;  # assumed i suppose :)
  }
  return( $list_unsafe, $errors, $fatal_error, $req_error );
}


## this is called on each field and runs the basic tests on each field
## depending on what the fieldname is or what type it is
sub validate_incoming_field {
  # this still needs a lot of work and needs to do santity checking on
  # most of the thigns in the database (it's only doing 6 or 7 at this
  # point
  my ($field) = @_;

  # check for tricky stuff like fields outside of what should passed
  # by the interface we've supplied.. return 1 which will cause a
  # fatal error and make it stop checking
  if ( ( length $field->{value} ) > $field->{size} )
    { return( $field, '1'); } # FATAL ERROR, do not pass go, do not collect..


  # make sure email fields are valid email addressese with this code
  if ( grep { $field->{name} eq $_ } qw( contactemail
	                                 owneremail moderatorbesidesemail)) {
    if ($field->{value} =~ m/\w+\@\w+.*\.\w{2,4}/) # nasty regex
                                                   # checks this but
                                                   # of course it
                                                   # could be a lot
                                                   # better
      { $field->{valid} = 1; return $field; } # mark an error
    else { return $field; }                   # return as a success
  }

  # webinfo and webarchive need to be urls
  elsif ( grep { $field->{name} eq $_ } qw( webinfo webarchive ) ) {
    if ( $field->{value} =~ m|^http://| ) 
      { $field->{valid} = 1;  return $field; }
    else { return $field; }
  }

  # this checks fields that can be EITHER email addresses or URL's
  elsif ( grep { $field->{name} eq $_ } qw( listadd
                                            subadd unsubadd ) ) {
    if ( $field->{value} =~ m/\w+\@\w+.*\.\w{2,4}/ # we've see this ugliness
	 || $field->{value} =~ m|^http://| )       # anything that
                                                   # starts with
                                                   # htp://, again,
                                                   # this could better
                                                   # but i've slacked
                                                   # on this because
                                                   # of time
      { $field->{valid} = 1;  return $field; }
    else { return $field; }
  }

  # numerical fields need to be pure digits
  elsif ( grep { $field->{name} eq $_ } qw( subcount msgsperweek ) ) {
    if ( $field->{value} =~ m/^\d+$/ ) 
      { $field->{valid} = 1;  return $field; }
    else { return $field; }
  }

  # zip codes need to be five digits and numbers.. check those here
  elsif ($field->{name} eq 'zip') {
    if ($field->{value} =~ m/\d{5}/) { $field->{valid} = 1; return $field; }
    else { return $field; }
  }

  # states need to be 2 valid states (they come from a select)
  elsif ($field->{name} eq 'state') {
    if ($field->{value} =~ m/[A-Z]{2}/) { $field->{valid} = 1; return $field; }
    else { return $field; }
  }

  # descriptions should be cleaned up first (never rejected)
  elsif ($field->{name} eq 'description') {
    $field->{value} =~ s/\s+/ /g;
    $field->{value} =~ s/^\s//;
    $field->{valid} = 1;
    return $field;
  }

  # in other cases, we trust it and move on, i doubt this is smart. :)
  else {
    $field->{valid} = 1;
    return $field;
  }
}

sub check_repeats {
  my ( $field, $value ) = @_;
  my $db_statement;
  my $list_id;

  $db_statement = qq/ SELECT id FROM calf_data WHERE $field = /;
  $db_statement .= ( $dbh->quote( $value ) );
  $db_statement .= q/ LIMIT 1 /;

  $sth = $dbh->prepare( $db_statement )
    || &db_error( "prepare", $!, $db_statement );
  $sth->execute
    || &db_error( "execute", $!, $db_statement );

  $list_id = ( $sth->fetchrow_array )[0];

  &error( 'insert', 'repeatlist', $value ) if ( defined $list_id  );
  $sth->finish || &db_error( 'finish', $! );
  return(1);

}
