package Calf::Utils;
# Utils.pm -- Mako Hill <mako@debian.org>
# written for Organizers' Collaborative -- 2001
# this code is licensed unde the GNU GPL, see COPYING for more details

# this module contains small utilities called by multiple distinct
# aspects (scripts) of the application. it is divided up into several
# major parts which correspond ot the export tages i've made available
# below. please read the comments in those sections for more info

# i should not be passing the database handle the way I am doing here
# now. it's not necessary and i think it's messy but it's too late to
# change right now

use Calf::Config qw/ :base :template_config %dbconfig /;
use CGI::Carp qw/ fatalsToBrowser set_message /;
use Template;                                    # templating system
use DBI;                                         # database abstraction layer

# this sets up a system where majorc compiliation failures and such
# will still render into the browser and not make ugly errors
BEGIN {
  sub code_error {
    my $error_message = shift;

   &process_template('error.html', { type => 'code', 
                                      time => scalar( localtime(time) ) } );
  }
  set_message(\&code_error);
}

use Exporter;
@ISA = ('Exporter');

# all the variables we'll make accessible outside this script
@EXPORT_OK = qw( &retrieve_type_codes
	         &make_type_hash &parse_codes &grab_listinfo
                 &build_descriptions
                 &process_template &process_cookie &code_error &dg_error
                 &open_db &db_error &send_email );

# the script is broken up into the following groups

%EXPORT_TAGS = (# all our base utils that everyone needs (there is
                # stuff in here that really should be moved out of
                # here, it's become a bit of a grab-bag unfortunately
		base => [ qw( &retrieve_type_codes
                              &make_type_hash &parse_codes &grab_listinfo
                              &build_descriptions &dg_error ) ],
		# tools that form the HTML templating system
		template => [ qw( &process_template &process_cookie
                                  &code_error ) ],
		# tools that form the database interface
		db => [ qw( &open_db &db_error ) ],
		# tools that help in emailing people more easily
		mail => [ qw( &send_email ) ]
	       );

use strict;

########################################################################
### base -- utilities which should be common across the board
########################################################################

## dg_error -- fail gracefully and print something nice about it in the
## error log to help debug it later
##### args: lots :) look below
sub dg_error ($$$$$) {
  # grab variables passed to it
  my( $type, $script, $subroutine, $reason, $error_details ) = @_;
  my $error_msg;

  # pass these variables, marked correctly, to the error.html template
  my $vars = { time => scalar( localtime(time) ),
	       type => $type,
	       script => $script,
               subroutine => $subroutine,
	       reason => $reason,
	       details => $error_details };
  &process_template( 'error.html', $vars );

  # be sure to print a useful warning in the error log so if people
  # report it, we can have an easier time debugging the problem
  $error_msg =  "a Calf user encountered an error on: ";
  $error_msg .= "( type: $type ) " if (defined $type );
  $error_msg .= "( script: $script ) " if ( defined $script );
  $error_msg .= "( sub: $subroutine ) " if ( defined $subroutine );
  $error_msg .= "( reason: $reason ) " if ( defined $reason );
  $error_msg .= "( error details: \"$error_details\" )"
    if ( defined $error_details );
  warn "$error_msg\n";
  exit;  # stop here, there's a dg_error, that should be the end of it
}


## build and return the status information which is displayed in the
## dyn_footer template
##### arg: the database handle
sub build_stats {
  my $dbh = &open_db;
  my ( $sth, $select_statement, %stats );

  $select_statement = q/ SELECT COUNT(*) FROM editors /;

  $sth = $dbh->prepare( $select_statement )
    || &db_error( "prepare", $!, $select_statement );
  $sth->execute
    || &db_error( "execute", $!, $select_statement );
  $stats{totaleditors} = ( $sth->fetchrow_array )[0];

  $select_statement = qq/ SELECT COUNT(*) FROM calf_data
                          WHERE recommended <= $rec_thresh /;
  $sth = $dbh->prepare( $select_statement )
    || &db_error( "prepare", $!, $select_statement );
  $sth->execute
    || &db_error( "execute", $!, $select_statement );
  $stats{totallists} = ( $sth->fetchrow_array )[0];

  $select_statement = qq/ SELECT COUNT(*) FROM calf_data
                          WHERE recommended <= $rec_thresh 
			  AND approved = 0 /;
  $sth = $dbh->prepare( $select_statement )
    || &db_error( "prepare", $!, $select_statement );
  $sth->execute
    || &db_error( "execute", $!, $select_statement );
  $stats{unapprovedlists} = ( $sth->fetchrow_array )[0];

  $sth->finish || &db_error( 'finish', $! );
  $dbh->disconnect;

  return ( \%stats );
}


## grab the codes for a particular issue or consituencies
##### arg: the raw codes, the type they fall in, and the codes
sub retrieve_type_codes ($$$) {
  my ( $raw_codes, $type, $dbh) = @_;
  my ( @codes, %code_hash, $categories, $code ) = ();

  # we grab a has of ALL the code => desc for the particular type
  $categories = &make_type_hash( $type, $dbh );

  # then we go through all the parsed out codes and make a list of
  # codes and a hash of the codes and their descriptions
  foreach $code (&parse_codes( $raw_codes ) ) {
    $code_hash{"$code"} = $categories->{$code};
    push @codes, $code;
  }
  # we return the same data in two ways, this also seems suspect to me
  # in forsight
  return( \%code_hash, @codes );
}

## parse the codes out into an array and return it
##### arguments: raw codes in a string
sub parse_codes ($) {
  my ($raw_codes) = @_;
  my @codes = ();

  # this is where my limit of 7 codes comes from.. there HAS to be a
  # abetter way to do this but one doesn't spring to mind and this
  # seems to work
  foreach (unpack "a2 a2 a2 a2 a2 a2 a2", $raw_codes) {
    push @codes, $_ if ($_ =~ m/\w{2}/);   # tack it onto the list
                                           # after we do a little bit
                                           # of error checking
  }
  return(@codes);
}

## take a type and a databse handle and return a hash with all the codes and
## their descriptions
##### arguments: $type and $dbh
sub make_type_hash ($$) {
  my ($type, $dbh) = @_;
  my $sth;
  my ( %categories ) = ();
  my ( $code, $description ) = ();

  # grab the contents of the table out of the db and then print stick
  # em in a hahs which we send out. this may no be as necessary once
  # we changed the table names but..
  $sth = $dbh->prepare(qq{ SELECT code, description FROM $type } )
    || &db_error( "prepare", $! );
  $sth->execute
    || &db_error( "execute", $! );
  while ( ( $code, $description ) = $sth->fetchrow_array) {
    $categories{$code} = $description;
  }
  return( \%categories );
}

## grab all the basic list information out of the database
##### arguments: item (list.id) and database handle
sub grab_listinfo ($$) {
  my ( $item, $dbh ) = @_;
  my ( $list, $field, $type, $sth, $select_statement );
  my $i;


  # put together the huge select statement
  $select_statement = q/ SELECT /;

  for ( $i = 0; $i <= $#all_fields; $i++ ) {
    $field = $all_fields[$i];
    $select_statement .= qq/ $field AS $field /;
    $select_statement .= "," unless $i == $#all_fields; 
  }

  $select_statement .= qq/ FROM calf_data
       	                   WHERE calf_data.id = "$item"
                           LIMIT 1 /;

  # prepare/execute the database statement
  $sth = $dbh->prepare( $select_statement )
    || &db_error( "prepare", $!, $select_statement );
  $sth->execute
    || &db_error( "execute", $!, $select_statement );

  # copy the data we've pulled into a NEW hashref by deferencing it
  # and then re-referncing the resulting hashed data
  $list = { %{ $sth->fetchrow_hashref('NAME_lc') } };

  $list->{description} =~ s/\s+/ /g;      # i clean up the descriptions a bit
  $list->{description} =~ s/^\s//;        # here so they show up ok in the
                                          # text areas

  # we should parse out the "types" into arrays so that we display
  # them more easily
  foreach $type (@types) {
    my @codes = &parse_codes( $list->{"$type"} );
    $list->{"$type"} = \@codes;
  }

  return( $list );  # return our data and that's all folks
}

## return a complex mash_type_hash for all types (as a hash indexed by type)
##### arg: just a database handle
sub build_descriptions ($) {
  my ( $dbh ) = @_;
  my ( $type, $descriptions );

  # this does EXACTLY what make_type_has does but it does it twice and
  # returns both in a single hash.. it's ugly but i used it alot.. ugh
  foreach $type (@types) {
    $descriptions->{"$type"} = &make_type_hash( $type, $dbh );
  }
  return( $descriptions );
}


########################################################################
### template -- utilities used to process templates and create pages
########################################################################
## process the template given with the variables given
##### arguments: $file, $vars
sub process_template ($$) {
  my ($file, $vars) = @_;

  $vars->{sitestats} = &build_stats;
  $vars->{calfwebroot} = $webserver_calfroot;

  print "Cache-Control: no-cache\n";
  # this of course is important so it will display in a
  # browser. however, this hsa caused some headaches because there's
  # already one printed when here are errors. it might be cool to know
  # this and leave it off in these cases
  print "Content-type: text/html\n\n";

  ## Set up the data for use by the templating system including the
  ## major options we'll need
  my $template = Template->new({
				INCLUDE_PATH => "$include_path",
				PRE_PROCESS  => "$config_file",
				POST_CHOMP   => 1,
				EVAL_PERL => 1	   # this i feel bad
                                                   # about but i use
                                                   # it one place
			       });
  ## process and build the page
  $template->process($file, $vars)
    || die $template->error(), "\n"; # this die should go and be
                                     # replaced with a pleasant error

}

## create and output a cookie with the session_id given.. it's a
## pretty straightfowrard script and it's documented in the CGI
## module's docs 
##### args: $session_id
sub process_cookie ($) {
  my $session_id = shift @_;
  my $q = new CGI;
  my $cookie = $q->cookie( -name => 'session_id',
			   -value => $session_id,
			   -path => '/',
			   -expires => '+59m' );
  print "Set-Cookie: $cookie\n";
}


########################################################################
### db -- utilities used to operate on the database
########################################################################
## open_db -- not much to this one, but it opens up a handle using the
## info in the config file
##### args: none
sub open_db {
  my $dbi_handle = DBI->connect( "dbi:$dbconfig{driver}:$dbconfig{name}",
				 "$dbconfig{username}", "$dbconfig{password}" );
  return $dbi_handle;
}

## handle DB errors gracefully in the browser window
#### args: the reasons code/short desc (prepare/select),
sub db_error ($$$) {
  my ( $reason, $error_msg, $statement ) = @_;

  # print out a pretty and elegant error using error.html
  my $vars = { type => 'db', 
               reason => $reason,
               time => scalar( localtime(time) ) };
  my $file = "error.html";
  &process_template( $file, $vars );

  # print out a custom error in the apache error.log
  warn "The software encountered encountered this fatal error: $error_msg\n\n";
  warn "Caused by this statement: $statement";
  exit;
}


########################################################################
### mail -- utilities used to deal with email
########################################################################
## send_mail -- does just what it sounds like it does. the way it's
## calling sendmial is not the way I'd like it at all at this point. i
## tried to move to the Mail::Mailer but with the version I tried, it
## would not let me mess with the From: field which is something that
## we will absolutely need.. so we're stuck doing it wrong way for now
##### args: \%headers, 'nameofthefile', \%varstohandtotemplate
sub send_email {
  my ( $headers, $file, $vars ) = @_;
  my ( $field, $value );

  unless ( $headers->{To} =~ m/\w+\@\w+.*\.\w{2,4}/ )
    { warn "APPROVE: email address invalid so email not sent";
      return; }

  # we need to open a handle to sendmail which on our machine is exim
  open( SENDMAIL, "|/usr/lib/sendmail -oi -t -odq")
    || &dg_error( 'mail', 'open', '', 'sendmail', $! );

  # we now need to iterate through the headers hash and print the headers
  while ( ( $field, $value ) = each( %$headers ) ) {
    print SENDMAIL "$field: $value\n";
  }
  print SENDMAIL "\n";  # the newline seperates headers from the body

  # we now open up a new template object with the output sub modified
  # so that it will send it to sendmail instead of STDOUT
  my $template = Template->new({ INCLUDE_PATH => "$include_path",
				 PRE_PROCESS  => "$config_file",
                                 OUTPUT       => sub { my $output = shift;
                                                       print SENDMAIL $output;
                                                     },
                                 POST_CHOMP   => 0,
				 EVAL_PERL    => 0                });
  ## process and build the email
  $template->process("mail/$file", $vars)
    || &dg_error( 'mail', '', '', 'template', $! );

  # now we close sendmail and are done with this little toad
  close( SENDMAIL ) 
    || &dg_error( 'mail', 'close', '', 'sendmail', $! );
}

1;
